package zero1hd.polyjet.ui.builders;

import java.io.IOException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.ui.Button;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.utils.Disposable;
import com.mpatric.mp3agic.ID3v2;
import com.mpatric.mp3agic.InvalidDataException;
import com.mpatric.mp3agic.Mp3File;
import com.mpatric.mp3agic.UnsupportedTagException;

import zero1hd.wavedecoder.WavInfo;

public class MusicSelectable extends Button implements Disposable {
	private boolean invalidMusic;
	private long durationInSeconds;
	private String songName;
	private Texture albumCover;
	WavInfo wavinfo;
	private byte[] albumWorkBytes;
	
	private volatile Image imageIcon;
	private Label displayName;
	private Label runTime;
	private Skin skin;
	
	public MusicSelectable(FileHandle musicFile, Preferences musicData, final Skin skin, Texture defaultAlbumC) {
		super(skin, "info-button");
		this.skin = skin;
		
		debug();
		
		this.albumCover = defaultAlbumC;
		
		if (musicFile.extension().toLowerCase().equals("mp3")) {
			try {
				Mp3File mp3File = new Mp3File(musicFile.file());
				durationInSeconds = mp3File.getLengthInSeconds();
				
				if (mp3File.hasId3v2Tag()) {
					ID3v2 id3v2tag = mp3File.getId3v2Tag();
					albumWorkBytes = id3v2tag.getAlbumImage();
					
					songName = id3v2tag.getTitle();
				}
			} catch (UnsupportedTagException | InvalidDataException | IOException e) {
				Gdx.app.debug("Music Selector", "Error while reading tag for " + musicFile.name());
			}
			
		} else {
			wavinfo = new WavInfo(musicFile.file());
			durationInSeconds = wavinfo.getDurationInSeconds();
		}
		if (durationInSeconds > 60 * 5) {
			invalidMusic = true;
		}
		
		if (songName == null || songName.isEmpty()) {
			songName = musicFile.nameWithoutExtension();
		}
	}
	
	public synchronized void addInfoToPanel() {
		Gdx.app.postRunnable(new Runnable() {
			
			@Override
			public void run() {
				if (albumWorkBytes != null && !invalidMusic) {
					Pixmap albumArt = new Pixmap(albumWorkBytes, 0, albumWorkBytes.length);
					albumCover = new Texture(albumArt);
					imageIcon = new Image(albumCover);
					albumArt.dispose();
				} else {
					imageIcon = new Image(albumCover);
					if (invalidMusic) {
						imageIcon.setColor(Color.RED);
					}
					System.out.println("invalid or null");
				}
				float scale = getWidth()/imageIcon.getWidth();
				imageIcon.setScale(scale);
//				imageIcon.setSize(scale*imageIcon.getWidth(), scale*imageIcon.getHeight());
				
				
				add(imageIcon);
				
				row();
				
				displayName = new Label(songName, skin);
				displayName.setWrap(true);
				add(displayName).prefWidth(getPrefWidth()-5).center();
				
				
				row();
				
				String formattedSecond = String.valueOf(durationInSeconds/60) + ":";
				
				if (durationInSeconds - (durationInSeconds/60)*60 < 10) {
					formattedSecond = formattedSecond.concat("0");
				}
				
				formattedSecond = formattedSecond.concat(String.valueOf(durationInSeconds - (durationInSeconds/60)*60));
				runTime = new Label(formattedSecond, skin);
				add(runTime);
			}
		});
	}
	
	@Override
	public void dispose() {
		albumCover.dispose();
	}
}
